'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
var nodePath = require('path');

exports.default = function (_ref) {
  var t = _ref.types;
  return {
    visitor: {
      Identifier: function Identifier(path, state) {
        if (path.node.name === '__filenamespace') {
          // Get user settings
          var _state$opts = state.opts,
              separator = _state$opts.separator,
              root = _state$opts.root,
              dropAllFilenames = _state$opts.dropAllFilenames;

          // Get file paths

          var _state$file$opts = state.file.opts,
              filename = _state$file$opts.filename,
              basename = _state$file$opts.basename,
              sourceRoot = _state$file$opts.sourceRoot;

          var dirname = nodePath.dirname(filename);

          // Get namespace root directory (defaults to projectRoot)
          // modified via user specified plugin option "root"
          var userSetRoot = root;
          var namespaceRoot = sourceRoot;
          if (userSetRoot) {
            // remove leading ./ (if it exists) as it adds no value
            var dotSlash = userSetRoot.match(/^\.\//);
            if (dotSlash) {
              userSetRoot = userSetRoot.replace(/^\.\//, '');
            }
            // travel up from project dir if userRoot starts with ../, please note
            // that all ../ path segments will be used and all others will be ignored
            // for example:
            // ../ === start from projectFolder
            // ../../ === start from projectParentFolder
            // ../folderA/../ === start from projectParentFolder (folderA is ignored)
            var dotDotSlash = userSetRoot.match(/^\.\.\//);
            if (dotDotSlash) {
              var sourcePathSegments = namespaceRoot.split('/');
              var rootPathSegments = userSetRoot.split('/');
              namespaceRoot = rootPathSegments.filter(function (val) {
                return val.match(/^\.\.$/);
              }).reduce(function (acc) {
                return acc.slice(0, -1);
              }, sourcePathSegments).join('/');
            }
          }

          // Set namespace root by stripping out unwanted path segments
          // example one:
          // If the babel plugin option root: '../' was set
          // "C:\Users\sam\myProject" would become "myProject"
          // example two:
          // If the babel plugin option root: './src' was set
          // "C:\Users\sam\myProject" would become "myProject/src"
          var filenamespace = dirname.replace(namespaceRoot, '');
          if (userSetRoot && filenamespace.startsWith('/' + userSetRoot)) {
            filenamespace = filenamespace.replace(userSetRoot, '');
          }

          // Add filename to namespace
          // Do not add filename if either:
          // - The `dropAllFilenames` plugin option has been set to true, OR
          // - Filename is "index" as it is meaningless
          var addFilename = !dropAllFilenames && !basename.match(/index/i);
          if (addFilename) {
            filenamespace += '/' + basename;
          }

          // Strip leading path separators
          filenamespace = filenamespace.replace(/^\/+/, '');

          // Set custom separator
          if (separator) {
            filenamespace = filenamespace.replace(/\//g, separator);
          }

          // Replace __filenamespace with the transformed value
          path.replaceWith(t.stringLiteral(filenamespace));
        }
      }
    }
  };
};